// This file is part of CycloneDX Go
//
// Licensed under the Apache License, Version 2.0 (the “License”);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an “AS IS” BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// SPDX-License-Identifier: Apache-2.0
// Copyright (c) OWASP Foundation. All Rights Reserved.

package cyclonedx

import (
	"bytes"
	"encoding/gob"
	"fmt"
)

// copy creates a deep copy of the BOM in a given destination.
// Copying is currently done by encoding and decoding the BOM struct using gob.
// In the future we may choose to switch to a more efficient strategy,
// and consider to export this API.
func (b BOM) copy(dst *BOM) error {
	buf := bytes.Buffer{}
	err := gob.NewEncoder(&buf).Encode(b)
	if err != nil {
		return fmt.Errorf("failed to encode bom: %w", err)
	}

	err = gob.NewDecoder(&buf).Decode(dst)
	if err != nil {
		return fmt.Errorf("failed to decode bom: %w", err)
	}

	return nil
}
